<?php

namespace Drupal\dblog_ban\Services;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Link;
use Drupal\Core\Routing\RedirectDestinationTrait;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Service to generate ban and unban links.
 */
class BanLinkGenerator implements ContainerInjectionInterface {
  use RedirectDestinationTrait;
  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static();
  }

  /**
   * Generate a ban link for an IP address.
   *
   * Note this function will happily generate a ban link for any IP address,
   * including the current user. If the current user bans themselves, they will
   * not be able to access the unban link anymore, and will have to use Drush or
   * direct database access to unban themselves. We recommend that you only call
   * this function if
   * \Drupal\dblog_ban\Services\IpValidator::isMyIp() === FALSE.
   *
   * @param string $ip
   *   The IP address to generate the link for.
   *
   * @return \Drupal\Core\Link
   *   A renderable ban link.
   */
  public function getBanLink(string $ip): Link {
    $url = $this->getBanUrl($ip);
    return Link::fromTextAndUrl($this->t('Ban @host', [
      '@host' => $ip,
    ]), $url);
  }

  /**
   * Generate a ban URL for an IP address.
   *
   * Note this function will happily generate a ban URL for any IP address,
   * including the current user. If the current user bans themselves, they will
   * not be able to access the unban link anymore, and will have to use Drush or
   * direct database access to unban themselves. We recommend that you only call
   * this function if
   * \Drupal\dblog_ban\Services\IpValidator::isMyIp() === FALSE.
   *
   * @param string $ip
   *   The IP address to generate the URL for.
   *
   * @return \Drupal\Core\Url
   *   A URL for a ban link.
   */
  public function getBanUrl(string $ip): Url {
    $urlIp = \urlencode($ip);
    $urlOptions = $this->getUrlOptions($ip, 'ban');
    return Url::fromRoute('dblog_ban.confirm_ban', [
      'ip' => $urlIp,
    ], $urlOptions);
  }

  /**
   * Generate an unban link for an IP address.
   *
   * While it is recommended to check whether
   * \Drupal\dblog_ban\Services\IpValidator::isMyIp() === FALSE before
   * displaying a ban link, the consequences of the current user unbanning
   * themselves is less serious: you don't necessarily need to check that before
   * calling this function.
   *
   * @param string $ip
   *   The IP address to generate the unban link for.
   *
   * @return \Drupal\Core\Link
   *   A renderable unban link.
   */
  public function getUnbanLink(string $ip): Link {
    $url = $this->getUnbanUrl($ip);
    return Link::fromTextAndUrl($this->t('Unban @host', [
      '@host' => $ip,
    ]), $url);
  }

  /**
   * Generate an unban URL for an IP address.
   *
   * While it is recommended to check whether
   * \Drupal\dblog_ban\Services\IpValidator::isMyIp() === FALSE before
   * displaying a ban link, the consequences of the current user unbanning
   * themselves is less serious: you don't necessarily need to check that before
   * calling this function.
   *
   * @param string $ip
   *   The IP address to generate the URL for.
   *
   * @return \Drupal\Core\Url
   *   A URL for an unban link.
   */
  public function getUnbanUrl(string $ip): Url {
    $urlIp = \urlencode($ip);
    $urlOptions = $this->getUrlOptions($ip, 'unban');
    return Url::fromRoute('dblog_ban.confirm_unban', [
      'ip' => $urlIp,
    ], $urlOptions);
  }

  /**
   * Get a set of URL options for a ban/unban URL.
   *
   * @param string $ip
   *   The IP address that we're generating a ban/unban URL for.
   * @param string $action
   *   The action to take, one of 'ban' or 'unban'.
   *
   * @return array
   *   An associative array of additional URL options. See
   *   \Drupal\Core\Url::fromUri() for the format.
   */
  protected function getUrlOptions(string $ip, string $action): array {
    $destinationQueryParam = $this->getDestinationArray();
    $htmlAttributes = [];

    return [
      'query' => $destinationQueryParam,
      'attributes' => $htmlAttributes,
    ];
  }

}
